package main

import (
  "log"
  "time"
  "github.com/gin-gonic/gin"
  "net/http"
  "encoding/json"
  "locsrv/database"
  "locsrv/mqtt"
)

/*
{
  "trigger":"enter",
  "device":"aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee",
  "device_model":"iPhone13,1",
  "timestamp":"1705162497.0",
  "latitude":"51.0",
  "id":"test",
  "longitude":"7.0",
  "device_type":"iPhone"
}
*/

type locativeEvent struct {
  Trigger string `json:"trigger"`
  Device string `json:"device"`
  Id string `json:"id,omitempty"`
  Location string `json:"location"`
  Latitude string `json:"latitude"`
  Longitude string `json:"longitude"`
  Person string `json:"person"`
  Timestamp string `json:"timestamp"`
}


func main() {
  router := gin.Default()
  ch := make(chan locativeEvent)
  dbh := database.New()
  mqtt := mqtt.New()

  go func() {
    for {
      select {
      case event := <- ch:
        person, err := dbh.GetPersonById(event.Device)
        event.Person = person
        event.Location = event.Id
        event.Timestamp = time.Now().Format("2006-01-02 15:04:05 MST")
        event.Id = ""
        if err != nil {
          log.Printf("Person unknown: %v", err)
        } 
        message, err2 := json.Marshal(event)
        if err2 != nil {
          log.Printf("Unable to marshal event: %v", err2)
        } else {
          log.Printf("Message: %s", message)
          err := mqtt.Publish(person, string(message))
          if err != nil {
            log.Printf("Failed to publish: %v", err)
          }
        }
      }
    }
  }()

  router.POST("/location", func(c *gin.Context) {
    err, data := c.GetRawData()
    log.Printf("raw: %v, %s", err, data)

    var body locativeEvent
    err1 := c.ShouldBindJSON(&body)
    if err1 != nil {
      c.JSON(http.StatusBadRequest, gin.H { "error": err1.Error() })
      return
    }

    var message string
    select {
    case ch <- body:
      message = "message forwarded"
    default:
      log.Println("Channel full, message lost")
      message = "message lost"
    }

    c.JSON(http.StatusOK, gin.H { "message": message })
  })

  router.Run(":8080")
}

